@@ -13,9 +13,21 @@
 #include "konqviewmanager.h"
 #include "konqsettingsxt.h"
 
+#ifdef KActivities_FOUND
+#include "activitymanager.h"
+#if QT_VERSION_MAJOR < 6
+#include <KActivities/Consumer>
+#else //QT_VERSION_MAJOR
+#include <PlasmaActivities/Consumer>
+#endif //QT_VERSION_MAJOR
+#endif //KActivities_FOUND
+
 #include "konqdebug.h"
 #include <kio/deletejob.h>
 #include <KLocalizedString>
+#include <KWindowInfo>
+#include <KX11Extras>
+
 #include <QUrl>
 #include <QIcon>
 #include <ksqueezedtextlabel.h>
@@ -35,13 +47,16 @@
 #include <QTreeWidget>
 #include <QScrollBar>
 #include <QApplication>
-#include <QDesktopWidget>
 #include <QStandardPaths>
 #include <QSessionManager>
-#include <KSharedConfig>
 #include <KConfigGroup>
 #include <QDialogButtonBox>
 #include <KGuiItem>
+#include <QScreen>
+#include <KSharedConfig>
+
+#include "konqapplication.h"
+#include <QMessageBox>
 
 class KonqSessionManagerPrivate
 {
@@ -101,7 +116,7 @@ SessionRestoreDialog::SessionRestoreDialog(const QStringList &sessionFilePaths,
         iconLayout->addWidget(iconLabel);
         iconLayout->addStretch(5);
         hLayout->addLayout(iconLayout, 0);
-        hLayout->addSpacing(style()->pixelMetric(QStyle::PM_DefaultLayoutSpacing));
+        hLayout->addSpacing(style()->pixelMetric(QStyle::PM_LayoutHorizontalSpacing));
     }
 
     const QString text(i18n("Konqueror did not close correctly. Would you like to restore these previous sessions?"));
@@ -122,17 +137,17 @@ SessionRestoreDialog::SessionRestoreDialog(const QStringList &sessionFilePaths,
     styleOption.initFrom(m_treeWidget);
     QFontMetrics fm(styleOption.font);
     int w = m_treeWidget->width();
-    const QRect desktop = QApplication::desktop()->screenGeometry(this);
+    const QRect desktop = screen()->geometry();
 
     // Collect info from the sessions to restore
-    Q_FOREACH (const QString &sessionFile, sessionFilePaths) {
+    for (const QString &sessionFile: sessionFilePaths) {
         qCDebug(KONQUEROR_LOG) << sessionFile;
         QTreeWidgetItem *windowItem = nullptr;
         KConfig config(sessionFile, KConfig::SimpleConfig);
         const QList<KConfigGroup> groups = windowConfigGroups(config);
-        Q_FOREACH (const KConfigGroup &group, groups) {
+        for (const KConfigGroup &group: groups) {
             // To avoid a recursive search, let's do linear search on Foo_CurrentHistoryItem=1
-            Q_FOREACH (const QString &key, group.keyList()) {
+            for (const QString &key: group.keyList()) {
                 if (key.endsWith(QLatin1String("_CurrentHistoryItem"))) {
                     const QString viewId = key.left(key.length() - qstrlen("_CurrentHistoryItem"));
                     const QString historyIndex = group.readEntry(key, QString());
@@ -347,6 +362,9 @@ KonqSessionManager::KonqSessionManager()
     , m_autosaveEnabled(false) // so that enableAutosave works
     , m_createdOwnedByDir(false)
     , m_sessionConfig(nullptr)
+#ifdef KActivities_FOUND
+    , m_activityManager(new ActivityManager(this))
+#endif
 {
     // Initialize dbus interfaces
     new KonqSessionManagerAdaptor(this);
@@ -529,10 +547,12 @@ void KonqSessionManager::saveCurrentSessionToFile(KConfig *config, const QList<K
         return;
     }
 
-    foreach (KonqMainWindow *window, mainWindows) {
+    for (KonqMainWindow *window: mainWindows) {
         if (!window->isPreloaded()) {
             KConfigGroup configGroup(config, "Window" + QString::number(counter));
             window->saveProperties(configGroup);
+            KWindowInfo info(window->winId(), NET::Properties(), NET::WM2Activities);
+            configGroup.writeEntry("Activities", info.activities());
             counter++;
         }
     }
@@ -619,7 +639,7 @@ QStringList KonqSessionManager::takeSessionsOwnership()
 void KonqSessionManager::restoreSessions(const QStringList &sessionFilePathsList,
         bool openTabsInsideCurrentWindow, KonqMainWindow *parent)
 {
-    foreach (const QString &sessionFilePath, sessionFilePathsList) {
+    for (const QString &sessionFilePath: sessionFilePathsList) {
         restoreSession(sessionFilePath, openTabsInsideCurrentWindow, parent);
     }
 }
@@ -644,7 +664,7 @@ void KonqSessionManager::restoreSession(const QString &sessionFilePath, bool
 
     KConfig config(sessionFilePath, KConfig::SimpleConfig);
     const QList<KConfigGroup> groups = windowConfigGroups(config);
-    Q_FOREACH (const KConfigGroup &configGroup, groups) {
+    for (const KConfigGroup &configGroup: groups) {
         if (!openTabsInsideCurrentWindow) {
             KonqViewManager::openSavedWindow(configGroup)->show();
         } else {
@@ -677,7 +697,7 @@ static void removeDiscardedSessions(const QStringList &sessionFiles, const QStri
         return;
     }
 
-    Q_FOREACH (const QString &sessionFile, sessionFiles) {
+    for (const QString &sessionFile: sessionFiles) {
         KConfig config(sessionFile, KConfig::SimpleConfig);
         QList<KConfigGroup> groups = windowConfigGroups(config);
         for (int i = 0, count = groups.count(); i < count; ++i) {
@@ -755,3 +775,19 @@ void KonqSessionManager::setPreloadedWindowsNumber(const QList<int> &numbers)
 {
     m_preloadedWindowsNumber = numbers;
 }
+
+void KonqSessionManager::registerMainWindow(KonqMainWindow* window)
+{
+#ifdef KActivities_FOUND
+    m_activityManager->registerMainWindow(window);
+#else
+    Q_UNUSED(window);
+#endif
+}
+
+#ifdef KActivities_FOUND
+ActivityManager * KonqSessionManager::activityManager()
+{
+    return m_activityManager;
+}
+#endif
