@@ -122,7 +122,10 @@ public:
     void openUrl(const QUrl &url,
                  const QString &locationBarURL,
                  const QString &nameFilter = QString(),
-                 bool tempFile = false);
+                 bool tempFile = false,
+                 const QUrl &requestedUrl={});
+
+    void duplicateView(KonqView *otherView);
 
     /**
      * Change the part inside this view if necessary.
@@ -243,9 +246,29 @@ public:
     void stop(bool keepTemporaryFile = false);
 
     /**
-     * Retrieve view's URL
+     * @brief The URL the view was requested to display
+     *
+     * This will be the same as the part's URL except when the URL was downloaded using
+     * a part's \link KonqInterfaces::DownloaderExtension DownloaderExtension\endlink. In this case, this will be the URL the user originally
+     * requested, while realUrl() is the URL of the downloaded file.
+     *
+     * @return the URL the view was requested to display
+     * @see realUrl()
+     * @see KonqInterfaces::DownloaderExtension
      */
-    QUrl url() const;
+    QUrl url() const {return m_requestedUrl;};
+
+    /**
+     * @brief The real URL displayed in the view
+     *
+     * If the URL represents a file which has been downloaded using a part's
+     * \link KonqInterfaces::DownloaderExtension DownloaderExtension\endlink before being displayed, then the real URL is the local
+     * file the URL was downloaded to. In all other cases, this is the same as url().
+     * @return the URL of the local file where a file was downloaded by a part's \link KonqInterfaces::DownloaderExtension DownloaderExtension\endlink
+     * or url() if the URL wasn't downloaded using \link KonqInterfaces::DownloaderExtension DownloaderExtension\endlink
+     * @see KonqInterfaces::DownloaderExtension
+     */
+    QUrl realUrl() const;
 
     QUrl upUrl() const;
 
@@ -358,6 +381,7 @@ public:
      */
     bool supportsMimeType(const QString &mimeType) const;
 
+    //TODO: it seems to be unused. Check and remove it
     /**
      * Whether the view in this part is suitable for web browsing.
      *
@@ -428,6 +452,18 @@ public:
         return m_bToggleView || m_bPassiveMode;
     }
 
+    /**
+     * @brief Whether the view can navigate to the given URL
+     *
+     * The view can always navigate to @p newUrl unless it's locked and @p newUrl is different
+     * from the current URL. If the view is following the current view, then the it can navigate to
+     * @p newUrl only if it's the same url as that of the current view
+     *
+     * @param newUrl the URL the view wants to navigate to
+     * @return `true` if the view can navigate to @p newUrl and `false` otherwise
+     */
+    bool canNavigateTo(const QUrl &newUrl) const;
+
     // True if "builtin" (see X-KDE-BrowserView-Built-Into)
     //bool isBuiltinView() const { return m_bBuiltinView; }
 
@@ -609,17 +645,27 @@ private Q_SLOTS:
     void slotCanceled(const QString &errMsg);
     void slotPercent(KJob *, unsigned long percent);
     void slotSpeed(KJob *, unsigned long bytesPerSecond);
+
+    //The four slots below are called in response to the several variants of the NavigationExtension::popupMenu signal.
+    //There are four variants of this signal: two come from KParts::NavigationExtension and two from BrowserExtension,
+    //which (as of KF6) has been moved to Konqueror. For each of them, there are two variants, one taking a KFileItemList
+    //and one taing a QUrl.
+    //
+    //Since these slots need to be disconnected, we can't just use a lambda
+#if QT_VERSION_MAJOR > 5
+    void slotBrowserPopupMenuFiles(const QPoint &global,
+        const KFileItemList &items, const KParts::OpenUrlArguments &args, const BrowserArguments &bargs,
+        KParts::NavigationExtension::PopupFlags flags, const KParts::NavigationExtension::ActionGroupMap &actionGroups);
+    void slotBrowserPopupMenuUrl(const QPoint &global,
+        const QUrl &url, mode_t mode, const KParts::OpenUrlArguments &arguments, const BrowserArguments &bargs,
+        KParts::NavigationExtension::PopupFlags flags, const KParts::NavigationExtension::ActionGroupMap &actionGroups);
+#endif
     void slotPopupMenuFiles(const QPoint &global,
-            const KFileItemList &items,
-            const KParts::OpenUrlArguments &args,
-            KParts::NavigationExtension::PopupFlags flags,
-            const KParts::NavigationExtension::ActionGroupMap &actionGroups);
+        const KFileItemList &items, const KParts::OpenUrlArguments &args, KParts::NavigationExtension::PopupFlags flags,
+        const KParts::NavigationExtension::ActionGroupMap &actionGroups);
     void slotPopupMenuUrl(const QPoint &global,
-                   const QUrl &url,
-                   mode_t mode,
-                   const KParts::OpenUrlArguments &arguments,
-                   KParts::NavigationExtension::PopupFlags flags,
-                   const KParts::NavigationExtension::ActionGroupMap &actionGroups);
+        const QUrl &url, mode_t mode, const KParts::OpenUrlArguments &arguments, KParts::NavigationExtension::PopupFlags flags,
+        const KParts::NavigationExtension::ActionGroupMap &actionGroups);
 
     /**
      * Connected to the NavigationExtension
@@ -741,6 +787,7 @@ private:
     QString m_caption;
     QString m_tempFile;
     QString m_dbusObjectPath;
+    QUrl m_requestedUrl;
 
 #if QT_VERSION_MAJOR < 6
 #ifdef KActivities_FOUND
