@@ -22,6 +22,9 @@
 #include "webenginepart_ext.h"
 #include "qtwebengine6compat.h"
 
+#include "libkonq_utils.h"
+#include "interfaces/browser.h"
+
 #include <QWebEngineCertificateError>
 #include <QWebEngineSettings>
 #include <QWebEngineProfile>
@@ -43,6 +46,7 @@
 #include <KPasswdServerClient>
 #include <KJobWidgets>
 #include <KPluginMetaData>
+#include <KIO/JobUiDelegateFactory>
 
 #include <QStandardPaths>
 #include <QScreen>
@@ -67,24 +71,24 @@
 using namespace KonqInterfaces;
 
 WebEnginePage::WebEnginePage(WebEnginePart *part, QWidget *parent)
-        : QWebEnginePage(KonqWebEnginePart::Profile::defaultProfile(), parent),
-         m_kioErrorCode(0),
-         m_ignoreError(false),
-         m_part(part),
-         m_passwdServerClient(new KPasswdServerClient)
+    : QWebEnginePage(KonqWebEnginePart::Profile::defaultProfile(), parent),
+        m_kioErrorCode(0),
+        m_ignoreError(false),
+        m_part(part),
+        m_passwdServerClient(new KPasswdServerClient)
 #ifndef REMOTE_DND_NOT_HANDLED_BY_WEBENGINE
-         , m_dropOperationTimer(new QTimer(this))
+        , m_dropOperationTimer(new QTimer(this))
 #endif
 {
-    if (view())
-        WebEngineSettings::self()->computeFontSizes(view()->logicalDpiY());
+if (view())
+    WebEngineSettings::self()->computeFontSizes(view()->logicalDpiY());
 
     //setForwardUnsupportedContent(true);
 
     connect(this, &QWebEnginePage::geometryChangeRequested,
             this, &WebEnginePage::slotGeometryChangeRequested);
-//    connect(this, SIGNAL(unsupportedContent(QNetworkReply*)),
-//            this, SLOT(slotUnsupportedContent(QNetworkReply*)));
+    //    connect(this, SIGNAL(unsupportedContent(QNetworkReply*)),
+    //            this, SLOT(slotUnsupportedContent(QNetworkReply*)));
     connect(this, &QWebEnginePage::featurePermissionRequested,
             this, &WebEnginePage::slotFeaturePermissionRequested);
     connect(this, &QWebEnginePage::loadFinished,
@@ -103,8 +107,8 @@ WebEnginePage::WebEnginePage(WebEnginePart *part, QWidget *parent)
     connect(this, &QWebEnginePage::certificateError, this, &WebEnginePage::handleCertificateError);
 #endif
 
-    //If this part is displaying the developer tools for another part, inform the other page it's not displaying the developer tools anymore.
-    //I'm not sure this is needed, but I think it's better to do it, just to be on the safe side
+//If this part is displaying the developer tools for another part, inform the other page it's not displaying the developer tools anymore.
+//I'm not sure this is needed, but I think it's better to do it, just to be on the safe side
     auto unsetInspectedPageIfNeeded = [this](bool ok) {
         if (ok && inspectedPage() && url().scheme() != QLatin1String("devtools")) {
             setInspectedPage(nullptr);
@@ -120,7 +124,7 @@ WebEnginePage::WebEnginePage(WebEnginePart *part, QWidget *parent)
 
 WebEnginePage::~WebEnginePage()
 {
-    //qCDebug(WEBENGINEPART_LOG) << this;
+//qCDebug(WEBENGINEPART_LOG) << this;
 }
 
 const WebSslInfo& WebEnginePage::sslInfo() const
@@ -151,8 +155,8 @@ static QString checkForDownloadManager(QWidget* widget)
     const QString exeName = QStandardPaths::findExecutable(fileName);
     if (exeName.isEmpty()) {
         KMessageBox::detailedError(widget,
-                                   i18n("The download manager (%1) could not be found in your installation.", fileName),
-                                   i18n("Try to reinstall it and make sure that it is available in $PATH. \n\nThe integration will be disabled."));
+                                    i18n("The download manager (%1) could not be found in your installation.", fileName),
+                                    i18n("Try to reinstall it and make sure that it is available in $PATH. \n\nThe integration will be disabled."));
         cfg.writePathEntry("DownloadManager", QString());
         cfg.sync();
         return QString();
@@ -175,8 +179,8 @@ bool WebEnginePage::downloadWithExternalDonwloadManager(const QUrl &url)
     const QString managerExe = QStandardPaths::findExecutable(fileName);
     if (managerExe.isEmpty()) {
         KMessageBox::detailedError(view(),
-                                   i18n("The download manager (%1) could not be found in your installation.", fileName),
-                                   i18n("Try to reinstall it and make sure that it is available in $PATH. \n\nThe integration will be disabled."));
+                                    i18n("The download manager (%1) could not be found in your installation.", fileName),
+                                    i18n("Try to reinstall it and make sure that it is available in $PATH. \n\nThe integration will be disabled."));
         cfg.writePathEntry("DownloadManager", QString());
         cfg.sync();
         return false;
@@ -189,7 +193,7 @@ bool WebEnginePage::downloadWithExternalDonwloadManager(const QUrl &url)
     return true;
 }
 
-void WebEnginePage::requestDownload(QWebEngineDownloadRequest *item, bool newWindow, bool requestSave)
+void WebEnginePage::requestDownload(QWebEngineDownloadRequest *item, bool newWindow, WebEnginePartDownloadManager::DownloadObjective objective)
 {
     QUrl url = item->url();
     if (downloadWithExternalDonwloadManager(url)) {
@@ -199,9 +203,9 @@ void WebEnginePage::requestDownload(QWebEngineDownloadRequest *item, bool newWin
     }
 
     WebEngineDownloaderExtension *downloader = m_part->downloader();
-    if (downloader) {
-        downloader->addDownloadRequest(item);
-    }
+    Q_ASSERT(downloader);
+
+    downloader->addDownloadRequest(item);
 
     BrowserArguments bArgs;
     bArgs.setForcesNewWindow(newWindow);
@@ -216,49 +220,102 @@ void WebEnginePage::requestDownload(QWebEngineDownloadRequest *item, bool newWin
 
     args.metaData().insert(QStringLiteral("DontSendToDefaultHTMLPart"), QString());
     args.metaData().insert(QStringLiteral("SuggestedFileName"), item->suggestedFileName());
-    if (requestSave) {
-        //The value doesn't matter
-        args.metaData().insert(QStringLiteral("ForceSave"), QString());
-    }
 
-//TODO KF6: this lamba only exists to reduce code duplication in the two #ifdef
-//branches below. When KF5 compatibility will be removed, cookies will always
-//be managed internally, so remove the #ifdef and the lambda and move its body
-//inside the `if (downloader)`
-    auto requestDownloadAndOpen = [&]() {
-        args.metaData().insert(QStringLiteral("TempFile"), QString());
-        emit downloader->downloadAndOpenUrl(url, item->id(), args, bArgs, true);
-        if (item->state() == QWebEngineDownloadRequest::DownloadRequested) {
-            qCDebug(WEBENGINEPART_LOG()) << "Automatically accepting download for" << item->url() << "This shouldn't happen";
-            item->accept();
-        }
-    };
-
-//TODO KF6: remove #ifndef and line inside it when compatibility with KF5 isn't needed anymore.
-#ifdef MANAGE_COOKIES_INTERNALLY
-    if (downloader) {
-        requestDownloadAndOpen();
-    } else { //It should never happen
-        qCDebug(WEBENGINEPART_LOG()) << "WebEnginePart for" << part()->url() << "doesn't have a WebEnginePartDownloaderExtension";
-#if QT_VERSION_MAJOR < 6
-        emit m_part->browserExtension()->openUrlRequest(url, args, bArgs);
-#else
-        emit m_part->browserExtension()->browserOpenUrlRequest(url, args, bArgs);
-#endif
-
-        item->cancel();
-        item->deleteLater();
-    }
-#else
-// blob URLs can't be downloaded by KIO, so use the part to download them even when using KCookieJar
-    if (item->url().scheme() == QStringLiteral("blob") && downloader) {
-        requestDownloadAndOpen();
+    if (objective == WebEnginePartDownloadManager::DownloadObjective::SaveOnly) {
+        args.metaData().insert(QStringLiteral("EmbedOrNothing"), {});
+        bArgs.setForcesNewWindow(true);
+        saveUrlToDiskAndDisplay(item, args, bArgs);
+        return;
+    } else if (objective == WebEnginePartDownloadManager::DownloadObjective::SaveAs) {
+        saveAs(item);
     } else {
-        emit m_part->browserExtension()->openUrlRequest(url, args, bArgs);
-        item->cancel();
-        item->deleteLater();
-    }
+    //TODO KF6: this lamba only exists to reduce code duplication in the two #ifdef
+    //branches below. When KF5 compatibility will be removed, cookies will always
+    //be managed internally, so remove the #ifdef and the lambda and move its body
+    //inside the `if (downloader)`
+        auto requestDownloadAndOpen = [&]() {
+            args.metaData().insert(QStringLiteral("TempFile"), QString());
+            emit downloader->downloadAndOpenUrl(url, item->id(), args, bArgs, true);
+            if (item->state() == QWebEngineDownloadRequest::DownloadRequested) {
+                qCDebug(WEBENGINEPART_LOG()) << "Automatically accepting download for" << item->url() << "This shouldn't happen";
+                item->accept();
+            }
+        };
+
+//TODO KF6: remove #ifdef and the #else block when dropping compatibility with KF5
+#ifdef MANAGE_COOKIES_INTERNALLY
+        requestDownloadAndOpen();
+#else //This can only happen in KF5, as MANAGE_COOKIES_INTERNALLY is always defined in KF6
+        // blob URLs can't be downloaded by KIO, so use the part to download them even when using KCookieJar
+        if (item->url().scheme() == QStringLiteral("blob") && downloader) {
+            requestDownloadAndOpen();
+        } else {
+            emit m_part->browserExtension()->openUrlRequest(url, args, bArgs);
+            item->cancel();
+            item->deleteLater();
+        }
 #endif
+    }
+}
+
+void WebEnginePage::saveUrlToDiskAndDisplay(QWebEngineDownloadRequest* req, const KParts::OpenUrlArguments& args, const BrowserArguments& bArgs)
+{
+    QWidget *window = view() ? view()->window() : nullptr;
+
+    QString suggestedName = !req->suggestedFileName().isEmpty() ? req->suggestedFileName() : req->url().fileName();
+    QString downloadPath = Konq::askDownloadLocation(suggestedName, window);
+    if (downloadPath.isEmpty()) {
+        req->cancel();
+        return;
+    }
+
+    WebEngineDownloaderExtension *downloader = m_part->downloader();
+    DownloaderJob *job = downloader->downloadJob(req->url(), req->id(), this);
+    if (!job) {
+        return;
+    }
+
+    auto lambda = [this, args, bArgs](DownloaderJob *, const QUrl &url) {
+    #if QT_VERSION_MAJOR < 6
+        emit m_part->browserExtension()->openUrlRequest(url, args, bArgs);
+    #else
+        emit m_part->browserExtension()->browserOpenUrlRequest(url, args, bArgs);
+    #endif
+    };
+    job->startDownload(downloadPath, window, this, lambda);
+}
+
+void WebEnginePage::saveAs(QWebEngineDownloadRequest* req)
+{
+    QWidget *window = view() ? view()->window() : nullptr;
+
+    QString suggestedName = !req->suggestedFileName().isEmpty() ? req->suggestedFileName() : req->url().fileName();
+    QString downloadPath = Konq::askDownloadLocation(suggestedName, window);
+    if (downloadPath.isEmpty()) {
+        req->cancel();
+        return;
+    }
+
+    WebEngineDownloaderExtension *downloader = m_part->downloader();
+    DownloaderJob *job = downloader->downloadJob(req->url(), req->id(), this);
+    if (!job) {
+        return;
+    }
+
+    auto lambda = [this](DownloaderJob *dj, const QUrl &url) {
+        if (dj->error() == 0) {
+            m_part->openUrl(url);
+            return;
+        }
+        BrowserArguments bArgs;
+        bArgs.setForcesNewWindow(true);
+#if QT_VERSION_MAJOR < 6
+        emit m_part->browserExtension()->openUrlRequest(url, {}, bArgs);
+#else
+        emit m_part->browserExtension()->browserOpenUrlRequest(url, {}, bArgs);
+#endif
+    };
+    job->startDownload(downloadPath, window, this, lambda);
 }
 
 # ifndef REMOTE_DND_NOT_HANDLED_BY_WEBENGINE
@@ -325,12 +382,18 @@ bool WebEnginePage::askBrowserToOpenUrl(const QUrl& url, const QString& mimetype
 #else
     emit m_part->browserExtension()->browserOpenUrlRequest(url, args, bargs);
 #endif
-    return true;
+    return false;
 }
 
-bool WebEnginePage::shouldOpenLocalUrl(const QUrl& url) const
+bool WebEnginePage::shouldOpenUrl(const QUrl& url) const
 {
-    Q_ASSERT(url.isLocalFile());
+    static const QStringList s_forcedSchemes{QStringLiteral("remote"), QStringLiteral("trash")};
+    if (s_forcedSchemes.contains(url.scheme())) {
+        return false;
+    }
+    if (!url.isLocalFile()) {
+        return true;
+    }
     BrowserInterface *bi = m_part->browserExtension()->browserInterface();
     bool useThisPart = false;
     //We don't check whether bi is valid, as invokeMethod will fail if it's nullptr
@@ -342,8 +405,16 @@ bool WebEnginePage::shouldOpenLocalUrl(const QUrl& url) const
 
 bool WebEnginePage::acceptNavigationRequest(const QUrl& url, NavigationType type, bool isMainFrame)
 {
-    if (isMainFrame && url.isLocalFile()) {
-        if (!shouldOpenLocalUrl(url)) {
+    //Ask the browser for permission to navigate away. In Konqueror, if a view is locked, it can't navigate to somewhere else
+    if (isMainFrame) {
+        KonqInterfaces::Browser *browser = KonqInterfaces::Browser::browser(qApp);
+        if (browser && !browser->canNavigateTo(part(), url)) {
+            return false;
+        }
+    }
+
+    if (isMainFrame) {
+        if (!shouldOpenUrl(url)) {
             return askBrowserToOpenUrl(url);
         }
     }
@@ -823,7 +894,11 @@ void WebEnginePage::setStatusBarText(const QString& text)
 
 void WebEnginePage::changeLifecycleState(QWebEnginePage::LifecycleState recommendedState)
 {
-    setLifecycleState(recommendedState == LifecycleState::Discarded ? LifecycleState::Frozen : recommendedState);
+    if (recommendedState != QWebEnginePage::LifecycleState::Active && !view()->isVisible()) {
+        setLifecycleState(QWebEnginePage::LifecycleState::Frozen);
+    } else {
+        setLifecycleState(QWebEnginePage::LifecycleState::Active);
+    }
 }
 
 void WebEnginePage::updateUserStyleSheet(const QString& script)
@@ -872,97 +947,77 @@ static BrowserArguments browserArgs(WebEnginePage::WebWindowType type)
     return bargs;
 }
 
+bool NewWindowPage::decideHandlingOfJavascripWindow(const QUrl url) const
+{
+    const HtmlSettingsInterface::JSWindowOpenPolicy policy = WebEngineSettings::self()->windowOpenPolicy(url.host());
+    switch (policy) {
+    case HtmlSettingsInterface::JSWindowOpenDeny:
+        // TODO: Implement support for dealing with blocked pop up windows.
+        return false;
+    case HtmlSettingsInterface::JSWindowOpenAsk: {
+        const QString message = (url.isEmpty() ?
+                                    i18n("This site is requesting to open a new popup window.\n"
+                                        "Do you want to allow this?") :
+                                    i18n("<qt>This site is requesting to open a popup window to"
+                                        "<p>%1</p><br/>Do you want to allow this?</qt>",
+                                        KStringHandler::rsqueeze(url.toDisplayString().toHtmlEscaped(), 100)));
+        return KMessageBox::questionTwoActions(view(), message, i18n("Javascript Popup Confirmation"),
+                                        KGuiItem(i18n("Allow")), KGuiItem(i18n("Do Not Allow"))) == KMessageBox::PrimaryAction;
+        // TODO: Implement support for dealing with blocked pop up windows.
+    }
+    default:
+        break;
+    }
+    return true;
+}
+
 bool NewWindowPage::acceptNavigationRequest(const QUrl &url, NavigationType type, bool isMainFrame)
 {
     //qCDebug(WEBENGINEPART_LOG) << "url:" << url << ", type:" << type << ", isMainFrame:" << isMainFrame << "m_createNewWindow=" << m_createNewWindow;
-    if (m_createNewWindow) {
-        const QUrl reqUrl (url);
-
-        const bool actionRequestedByUser = type != QWebEnginePage::NavigationTypeOther;
-        const bool actionRequestsNewTab = m_type == QWebEnginePage::WebBrowserBackgroundTab ||
-                                          m_type == QWebEnginePage::WebBrowserTab;
-
-        if (actionRequestedByUser && !actionRequestsNewTab) {
-            if (!part() && !isMainFrame) {
-                return false;
-            }
-            const HtmlSettingsInterface::JSWindowOpenPolicy policy = WebEngineSettings::self()->windowOpenPolicy(reqUrl.host());
-            switch (policy) {
-            case HtmlSettingsInterface::JSWindowOpenDeny:
-                // TODO: Implement support for dealing with blocked pop up windows.
-                this->deleteLater();
-                return false;
-            case HtmlSettingsInterface::JSWindowOpenAsk: {
-                const QString message = (reqUrl.isEmpty() ?
-                                          i18n("This site is requesting to open a new popup window.\n"
-                                               "Do you want to allow this?") :
-                                          i18n("<qt>This site is requesting to open a popup window to"
-                                               "<p>%1</p><br/>Do you want to allow this?</qt>",
-                                               KStringHandler::rsqueeze(reqUrl.toDisplayString().toHtmlEscaped(), 100)));
-                if (KMessageBox::questionTwoActions(view(), message,
-                                               i18n("Javascript Popup Confirmation"),
-                                               KGuiItem(i18n("Allow")),
-                                               KGuiItem(i18n("Do Not Allow"))) == KMessageBox::SecondaryAction) {
-                    // TODO: Implement support for dealing with blocked pop up windows.
-                    this->deleteLater();
-                    return false;
-                }
-               break;
-            }
-            default:
-                break;
-            }
-        }
-
-        // Browser args...
-        BrowserArguments bargs = browserArgs(m_type);
-
-        // OpenUrl args...
-        KParts::OpenUrlArguments uargs;
-        uargs.setMimeType(QL1S("text/html"));
-        uargs.setActionRequestedByUser(actionRequestedByUser);
-
-        // Window args...
-        WindowArgs wargs (m_windowArgs);
-
-        KParts::ReadOnlyPart* newWindowPart =nullptr;
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-        emit part()->browserExtension()->createNewWindow(QUrl(), uargs, bargs, wargs, &newWindowPart);
-#else
-        emit part()->browserExtension()->browserCreateNewWindow(QUrl(), uargs, bargs, wargs, &newWindowPart);
-#endif
-        qCDebug(WEBENGINEPART_LOG) << "Created new window" << newWindowPart;
-
-        if (!newWindowPart) {
-            return false;
-        } else if (newWindowPart->widget()->topLevelWidget() != part()->widget()->topLevelWidget()) {
-            KParts::OpenUrlArguments args;
-            args.metaData().insert(QL1S("new-window"), QL1S("true"));
-            newWindowPart->setArguments(args);
-        }
-
-        // Get the webview...
-        WebEnginePart* webenginePart = qobject_cast<WebEnginePart*>(newWindowPart);
-        WebEngineView* webView = webenginePart ? qobject_cast<WebEngineView*>(webenginePart->view()) : nullptr;
-
-        // If the newly created window is NOT a webenginepart...
-        if (!webView) {
-            qCDebug(WEBENGINEPART_LOG) << "Opening URL on" << newWindowPart;
-            newWindowPart->openUrl(reqUrl);
-            this->deleteLater();
-            return false;
-        }
-        webenginePart->setPage(this);
-        //Set the create new window flag to false...
-        m_createNewWindow = false;
-        if (webenginePart) {
-            QTimer::singleShot(0, webenginePart, [webenginePart, url](){emit webenginePart->navigationExtension()->openUrlRequest(url);});
-            return false;
-        }
-
+    if (!m_createNewWindow) {
+        return WebEnginePage::acceptNavigationRequest(url, type, isMainFrame);
     }
 
-    return WebEnginePage::acceptNavigationRequest(url, type, isMainFrame);
+    const QUrl reqUrl (url);
+    const bool actionRequestedByUser = type != QWebEnginePage::NavigationTypeOther;
+    const bool actionRequestsNewTab = m_type == QWebEnginePage::WebBrowserBackgroundTab || m_type == QWebEnginePage::WebBrowserTab;
+
+    if (actionRequestedByUser && !actionRequestsNewTab) {
+        if (!part() && !isMainFrame) {
+            return false;
+        }
+        if (!decideHandlingOfJavascripWindow(reqUrl)) {
+            deleteLater();
+            return false;
+        }
+    }
+
+    // Browser args...
+    BrowserArguments bargs = browserArgs(m_type);
+
+    // OpenUrl args...
+    KParts::OpenUrlArguments uargs;
+    uargs.setMimeType(QL1S("text/html"));
+    uargs.setActionRequestedByUser(actionRequestedByUser);
+
+    // Window args...
+    WindowArgs wargs (m_windowArgs);
+
+    KParts::ReadOnlyPart* newWindowPart = nullptr;
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    emit part()->browserExtension()->createNewWindow(url, uargs, bargs, wargs, &newWindowPart);
+#else
+    emit part()->browserExtension()->browserCreateNewWindow(url, uargs, bargs, wargs, &newWindowPart);
+#endif
+    qCDebug(WEBENGINEPART_LOG) << "Created new window" << newWindowPart;
+
+    if (newWindowPart && newWindowPart->widget()->topLevelWidget() != part()->widget()->topLevelWidget()) {
+        KParts::OpenUrlArguments args;
+        args.metaData().insert(QL1S("new-window"), QL1S("true"));
+        newWindowPart->setArguments(args);
+    }
+    deleteLater();
+    return false;
 }
 
 void NewWindowPage::slotGeometryChangeRequested(const QRect & rect)
